@@ -1,6 +1,6 @@
 # encoding: utf-8
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 import os
 import sys
@@ -9,9 +9,8 @@ from itertools import chain
 from six import iterkeys, iteritems
 from six.moves.configparser import ConfigParser
 
-from bpython.keys import cli_key_dispatch as key_dispatch
-from bpython.autocomplete import SIMPLE as default_completion
-import bpython.autocomplete
+from .autocomplete import SIMPLE as default_completion, ALL_MODES
+from .keys import cli_key_dispatch as cli_key_dispatch
 
 
 class Struct(object):
@@ -33,7 +32,7 @@ def can_encode(c):
 
 
 def supports_box_chars():
-    """Check if the encoding suppors Unicode box characters."""
+    """Check if the encoding supports Unicode box characters."""
     return all(map(can_encode, u'│─└┘┌┐'))
 
 
@@ -72,6 +71,7 @@ def loadini(struct, configfile):
             'color_scheme': 'default',
             'complete_magic_methods': True,
             'dedent_after': 1,
+            'default_autoreload': False,
             'editor': os.environ.get('VISUAL', os.environ.get('EDITOR', 'vi')),
             'flush_output': True,
             'highlight_show_source': True,
@@ -177,6 +177,7 @@ def loadini(struct, configfile):
     struct.hist_length = config.getint('general', 'hist_length')
     struct.hist_duplicates = config.getboolean('general', 'hist_duplicates')
     struct.flush_output = config.getboolean('general', 'flush_output')
+    struct.default_autoreload = config.getboolean('general', 'default_autoreload')
 
     struct.pastebin_key = get_key_no_doublebind('pastebin')
     struct.copy_clipboard_key = get_key_no_doublebind('copy_clipboard')
@@ -273,7 +274,7 @@ def loadini(struct, configfile):
     struct.hist_file = os.path.expanduser(struct.hist_file)
 
     # verify completion mode
-    if struct.autocomplete_mode not in bpython.autocomplete.ALL_MODES:
+    if struct.autocomplete_mode not in ALL_MODES:
         struct.autocomplete_mode = default_completion
 
     # set box drawing characters
