@@ -1,22 +1,27 @@
-import collections
-from itertools import islice
-import os
-import socket
-from six.moves import range
+# encoding: utf-8
 
-try:
-    import unittest2 as unittest
-except ImportError:
-    import unittest
+from itertools import islice
+from six.moves import range
+import collections
+import inspect
+import os
+import shutil
+import socket
+import sys
+import tempfile
 
 from bpython._py3compat import py3
 from bpython import config, repl, cli, autocomplete
-from bpython.test import MagicIterMock, mock
+from bpython.test import MagicIterMock, mock, FixLanguageTestCase as TestCase
+from bpython.test import unittest, TEST_CONFIG
+
+
+pypy = 'PyPy' in sys.version
 
 
 def setup_config(conf):
     config_struct = config.Struct()
-    config.loadini(config_struct, os.devnull)
+    config.loadini(config_struct, TEST_CONFIG)
     if 'autocomplete_mode' in conf:
         config_struct.autocomplete_mode = conf['autocomplete_mode']
     return config_struct
@@ -62,7 +67,7 @@ class TestMatchesIterator(unittest.TestCase):
             next(self.matches_iterator)
 
         self.assertEqual(next(self.matches_iterator), self.matches[0])
-        self.assertEqual(next(self.matches_iterator), self. matches[1])
+        self.assertEqual(next(self.matches_iterator), self.matches[1])
         self.assertNotEqual(next(self.matches_iterator), self.matches[1])
 
     def test_previous(self):
@@ -88,7 +93,8 @@ class TestMatchesIterator(unittest.TestCase):
         self.assertEqual(list(slice), self.matches * 3)
 
     def test_current(self):
-        self.assertRaises(ValueError, self.matches_iterator.current)
+        with self.assertRaises(ValueError):
+            self.matches_iterator.current()
         next(self.matches_iterator)
         self.assertEqual(self.matches_iterator.current(), self.matches[0])
 
@@ -113,7 +119,8 @@ class TestMatchesIterator(unittest.TestCase):
             self.matches_iterator.orig_line)
         self.matches_iterator.completer = completer
 
-        self.assertRaises(ValueError, self.matches_iterator.cur_line)
+        with self.assertRaises(ValueError):
+            self.matches_iterator.cur_line()
 
         self.assertEqual(next(self.matches_iterator), self.matches[0])
         self.assertEqual(self.matches_iterator.cur_line(),
@@ -133,6 +140,14 @@ class TestArgspec(unittest.TestCase):
         self.repl.push("    def spam(self, a, b, c):\n", False)
         self.repl.push("        pass\n", False)
         self.repl.push("\n", False)
+        self.repl.push("class SpammitySpam(object):\n", False)
+        self.repl.push("    def __init__(self, a, b, c):\n", False)
+        self.repl.push("        pass\n", False)
+        self.repl.push("\n", False)
+        self.repl.push("class WonderfulSpam(object):\n", False)
+        self.repl.push("    def __new__(self, a, b, c):\n", False)
+        self.repl.push("        pass\n", False)
+        self.repl.push("\n", False)
         self.repl.push("o = Spam()\n", False)
         self.repl.push("\n", False)
 
@@ -157,7 +172,6 @@ class TestArgspec(unittest.TestCase):
             self.assertTrue(self.repl.get_args())
             self.assertEqual(self.repl.current_func.__name__, expected_name)
 
-
     def test_syntax_error_parens(self):
         for line in ["spam(]", "spam([)", "spam())"]:
             self.set_input_line(line)
@@ -167,22 +181,22 @@ class TestArgspec(unittest.TestCase):
     def test_kw_arg_position(self):
         self.set_input_line("spam(a=0")
         self.assertTrue(self.repl.get_args())
-        self.assertEqual(self.repl.argspec[3], "a")
+        self.assertEqual(self.repl.arg_pos, "a")
 
         self.set_input_line("spam(1, b=1")
         self.assertTrue(self.repl.get_args())
-        self.assertEqual(self.repl.argspec[3], "b")
+        self.assertEqual(self.repl.arg_pos, "b")
 
         self.set_input_line("spam(1, c=2")
         self.assertTrue(self.repl.get_args())
-        self.assertEqual(self.repl.argspec[3], "c")
+        self.assertEqual(self.repl.arg_pos, "c")
 
     def test_lambda_position(self):
         self.set_input_line("spam(lambda a, b: 1, ")
         self.assertTrue(self.repl.get_args())
-        self.assertTrue(self.repl.argspec)
+        self.assertTrue(self.repl.funcprops)
         # Argument position
-        self.assertEqual(self.repl.argspec[3], 1)
+        self.assertEqual(self.repl.arg_pos, 1)
 
     def test_issue127(self):
         self.set_input_line("x=range(")
@@ -203,6 +217,52 @@ class TestArgspec(unittest.TestCase):
         self.set_input_line("spamspamspam(")
         self.assertFalse(self.repl.get_args())
 
+    def test_issue572(self):
+        self.set_input_line("SpammitySpam(")
+        self.assertTrue(self.repl.get_args())
+
+        self.set_input_line("WonderfulSpam(")
+        self.assertTrue(self.repl.get_args())
+
+    @unittest.skipIf(pypy, 'pypy pydoc doesn\'t have this')
+    def test_issue583(self):
+        self.repl = FakeRepl()
+        self.repl.push("a = 1.2\n", False)
+        self.set_input_line("a.is_integer(")
+        self.repl.set_docstring()
+        self.assertIsNot(self.repl.docstring, None)
+
+    def test_methods_of_expressions(self):
+        self.set_input_line("'a'.capitalize(")
+        self.assertTrue(self.repl.get_args())
+
+        self.set_input_line("(1 + 1.1).as_integer_ratio(")
+        self.assertTrue(self.repl.get_args())
+
+
+class TestArgspecInternal(unittest.TestCase):
+    def test_function_expressions(self):
+        te = self.assertTupleEqual
+        fa = lambda line: repl.Repl._funcname_and_argnum(line)
+        for line, (func, argnum) in [
+            ('spam(', ('spam', 0)),
+            ('spam((), ', ('spam', 1)),
+            ('spam.eggs((), ', ('spam.eggs', 1)),
+            ('spam[abc].eggs((), ', ('spam[abc].eggs', 1)),
+            ('spam[0].eggs((), ', ('spam[0].eggs', 1)),
+            ('spam[a + b]eggs((), ', ('spam[a + b]eggs', 1)),
+            ('spam().eggs((), ', ('spam().eggs', 1)),
+            ('spam(1, 2).eggs((), ', ('spam(1, 2).eggs', 1)),
+            ('spam(1, f(1)).eggs((), ', ('spam(1, f(1)).eggs', 1)),
+            ('[0].eggs((), ', ('[0].eggs', 1)),
+            ('[0][0]((), {}).eggs((), ', ('[0][0]((), {}).eggs', 1)),
+            ('a + spam[0].eggs((), ', ('spam[0].eggs', 1)),
+            ("spam(", ("spam", 0)),
+            ("spam(map([]", ("map", 0)),
+            ("spam((), ", ("spam", 1))
+                ]:
+            te(fa(line), (func, argnum))
+
 
 class TestGetSource(unittest.TestCase):
     def setUp(self):
@@ -215,31 +275,35 @@ class TestGetSource(unittest.TestCase):
 
     def assert_get_source_error_for_current_function(self, func, msg):
         self.repl.current_func = func
-        self.assertRaises(repl.SourceNotFound,
-                          self.repl.get_source_of_current_name)
+        with self.assertRaises(repl.SourceNotFound):
+            self.repl.get_source_of_current_name()
         try:
             self.repl.get_source_of_current_name()
         except repl.SourceNotFound as e:
             self.assertEqual(e.args[0], msg)
+        else:
+            self.fail("Should have raised SourceNotFound")
 
     def test_current_function(self):
         self.set_input_line('INPUTLINE')
-        self.repl.current_func = collections.MutableSet.add
-        self.assertIn("Add an element.",
+        self.repl.current_func = inspect.getsource
+        self.assertIn("text of the source code",
                       self.repl.get_source_of_current_name())
 
-        self.assert_get_source_error_for_current_function(
-            collections.defaultdict.copy, "No source code found for INPUTLINE")
-
-        self.assert_get_source_error_for_current_function(
-            collections.defaultdict, "could not find class definition")
-
         self.assert_get_source_error_for_current_function(
             [], "No source code found for INPUTLINE")
 
         self.assert_get_source_error_for_current_function(
             list.pop, "No source code found for INPUTLINE")
 
+    @unittest.skipIf(pypy, 'different errors for PyPy')
+    def test_current_function_cpython(self):
+        self.set_input_line('INPUTLINE')
+        self.assert_get_source_error_for_current_function(
+            collections.defaultdict.copy, "No source code found for INPUTLINE")
+        self.assert_get_source_error_for_current_function(
+            collections.defaultdict, "could not find class definition")
+
     def test_current_line(self):
         self.repl.interp.locals['a'] = socket.socket
         self.set_input_line('a')
@@ -248,6 +312,25 @@ class TestGetSource(unittest.TestCase):
 # TODO add tests for various failures without using current function
 
 
+class TestEditConfig(TestCase):
+    def setUp(self):
+        self.repl = FakeRepl()
+        self.repl.interact.confirm = lambda msg: True
+        self.repl.interact.notify = lambda msg: None
+        self.repl.config.editor = 'true'
+
+    def test_create_config(self):
+        tmp_dir = tempfile.mkdtemp()
+        try:
+            config_path = os.path.join(tmp_dir, 'newdir', 'config')
+            self.repl.config.config_path = config_path
+            self.repl.edit_config()
+            self.assertTrue(os.path.exists(config_path))
+        finally:
+            shutil.rmtree(tmp_dir)
+            self.assertFalse(os.path.exists(config_path))
+
+
 class TestRepl(unittest.TestCase):
 
     def set_input_line(self, line):
@@ -284,25 +367,25 @@ class TestRepl(unittest.TestCase):
                          ['def', 'del', 'delattr(', 'dict(', 'dir(',
                           'divmod('])
 
-    @unittest.skip("disabled while non-simple completion is disabled")
     def test_substring_global_complete(self):
         self.repl = FakeRepl({'autocomplete_mode': autocomplete.SUBSTRING})
         self.set_input_line("time")
 
         self.assertTrue(self.repl.complete())
-        self.assertTrue(hasattr(self.repl.completer, 'matches'))
-        self.assertEqual(self.repl.completer.matches,
+        self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
+        self.assertEqual(self.repl.matches_iter.matches,
                          ['RuntimeError(', 'RuntimeWarning('])
 
-    @unittest.skip("disabled while non-simple completion is disabled")
     def test_fuzzy_global_complete(self):
         self.repl = FakeRepl({'autocomplete_mode': autocomplete.FUZZY})
         self.set_input_line("doc")
 
         self.assertTrue(self.repl.complete())
-        self.assertTrue(hasattr(self.repl.completer, 'matches'))
-        self.assertEqual(self.repl.completer.matches,
-                         ['UnboundLocalError(', '__doc__'])
+        self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
+        self.assertEqual(self.repl.matches_iter.matches,
+                         ['UnboundLocalError(', '__doc__'] if not py3 else
+                         ['ChildProcessError(', 'UnboundLocalError(',
+                          '__doc__'])
 
     # 2. Attribute tests
     def test_simple_attribute_complete(self):
@@ -317,7 +400,6 @@ class TestRepl(unittest.TestCase):
         self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
         self.assertEqual(self.repl.matches_iter.matches, ['Foo.bar'])
 
-    @unittest.skip("disabled while non-simple completion is disabled")
     def test_substring_attribute_complete(self):
         self.repl = FakeRepl({'autocomplete_mode': autocomplete.SUBSTRING})
         self.set_input_line("Foo.az")
@@ -327,10 +409,9 @@ class TestRepl(unittest.TestCase):
             self.repl.push(line)
 
         self.assertTrue(self.repl.complete())
-        self.assertTrue(hasattr(self.repl.completer, 'matches'))
-        self.assertEqual(self.repl.completer.matches, ['Foo.baz'])
+        self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
+        self.assertEqual(self.repl.matches_iter.matches, ['Foo.baz'])
 
-    @unittest.skip("disabled while non-simple completion is disabled")
     def test_fuzzy_attribute_complete(self):
         self.repl = FakeRepl({'autocomplete_mode': autocomplete.FUZZY})
         self.set_input_line("Foo.br")
@@ -340,10 +421,10 @@ class TestRepl(unittest.TestCase):
             self.repl.push(line)
 
         self.assertTrue(self.repl.complete())
-        self.assertTrue(hasattr(self.repl.completer, 'matches'))
-        self.assertEqual(self.repl.completer.matches, ['Foo.bar'])
+        self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
+        self.assertEqual(self.repl.matches_iter.matches, ['Foo.bar'])
 
-    # 3. Edge Cases
+    # 3. Edge cases
     def test_updating_namespace_complete(self):
         self.repl = FakeRepl({'autocomplete_mode': autocomplete.SIMPLE})
         self.set_input_line("foo")
@@ -360,6 +441,20 @@ class TestRepl(unittest.TestCase):
         self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
         self.assertNotIn('__file__', self.repl.matches_iter.matches)
 
+    # 4. Parameter names
+    def test_paremeter_name_completion(self):
+        self.repl = FakeRepl({'autocomplete_mode': autocomplete.SIMPLE})
+        self.set_input_line("foo(ab")
+
+        code = "def foo(abc=1, abd=2, xyz=3):\n\tpass\n"
+        for line in code.split("\n"):
+            self.repl.push(line)
+
+        self.assertTrue(self.repl.complete())
+        self.assertTrue(hasattr(self.repl.matches_iter, 'matches'))
+        self.assertEqual(self.repl.matches_iter.matches,
+                         ['abc=', 'abd=', 'abs('])
+
 
 class TestCliRepl(unittest.TestCase):
 
@@ -403,7 +498,8 @@ class TestCliReplTab(unittest.TestCase):
         self.repl.print_line = mock.Mock()
         self.repl.matches_iter.is_cseq.return_value = False
         self.repl.show_list = mock.Mock()
-        self.repl.argspec = mock.Mock()
+        self.repl.funcprops = mock.Mock()
+        self.repl.arg_pos = mock.Mock()
         self.repl.matches_iter.cur_line.return_value = (None, "foobar")
 
         self.repl.s = "foo"
@@ -446,7 +542,8 @@ class TestCliReplTab(unittest.TestCase):
         self.repl.matches_iter.previous.return_value = "previtem"
         self.repl.matches_iter.is_cseq.return_value = False
         self.repl.show_list = mock.Mock()
-        self.repl.argspec = mock.Mock()
+        self.repl.funcprops = mock.Mock()
+        self.repl.arg_pos = mock.Mock()
         self.repl.matches_iter.cur_line.return_value = (None, "previtem")
         self.repl.print_line = mock.Mock()
         self.repl.s = "foo"
